import socket
import time
from pytest import Function, Session
from _pytest.reports import TestReport
from urllib import parse as urlparse
import requests


def process_request(method, url, **kwargs):
    try:
        return requests.request(method, url, timeout=3, **kwargs)
    except Exception as e:  # 不打断用例执行
        pass


class DataAPI(object):
    def __init__(self, api_host, session_id, agent_name, agent_ip):
        self.api_host = api_host
        self.report_url = urlparse.urljoin(api_host, 'report')
        self.result_url = urlparse.urljoin(api_host, 'result')
        self.session_id = session_id
        self.agent_name = agent_name
        self.agent_ip = agent_ip
        self.report_created = False

    def submit_test_cases(self, session: Session):
        # 提交session信息,session_id, project_name,total...
        resp = process_request('post', url=self.report_url, json={
            "id": self.session_id,
            "project_name": session.config.option.project_name,
            "total": session.testscollected,
            "success": 0,
            "fail": 0,
            "host_name": self.agent_name,
            "host_ip": self.agent_ip
        })
        if resp.status_code == 200:
            self.report_created = True
            # 初始化用例提交
            for case in session.items:
                case: Function
                name = case.name
                node_id = case.nodeid
                own_markers = case.own_markers
                marker_names = '|'.join(list(map(lambda m: m.name, own_markers)))
                zen_tao = list(filter(lambda m: m.name == 'allure_link' and m.kwargs['link_type'] == 'test_case', own_markers))
                if zen_tao:
                    zen_tao_id = zen_tao[0].args[0]
                else:
                    zen_tao_id = None  # 如果没有zen_tao_id值则提交0
                # todo 提交数据

                process_request('post', url=self.result_url, json={
                    "report_id": self.session_id,
                    "zentao_id": zen_tao_id,
                    "case_name": name,
                    "case_node_id": node_id.replace('/', '__'),
                    "mark_names": marker_names
                })

    def update_test_status(self, report: TestReport):
        if self.report_created:
            when = report.when
            node_id = report.nodeid
            cap_log = report.caplog
            cap_stdout = report.capstdout
            cap_stderr = report.capstderr
            long_repr_text = report.longreprtext
            outcome = report.outcome
            result_update_url = urlparse.urljoin(self.api_host, f'result/{self.session_id}/{node_id.replace("/", "__")}')
            payload = {'when': when}
            if when == 'setup':
                payload.update({
                    'setup_cap_log': cap_log,
                    'setup_cap_stdout': cap_stdout,
                    'setup_cap_stderr': cap_stderr,
                    'setup_long_repr_text': long_repr_text,
                    'setup_outcome': outcome
                })
            elif when == 'call':
                payload.update({
                    'call_cap_log': cap_log,
                    'call_cap_stdout': cap_stdout,
                    'call_cap_stderr': cap_stderr,
                    'call_long_repr_text': long_repr_text,
                    'call_outcome': outcome
                })
            else:
                payload.update({
                    'teardown_cap_log': cap_log,
                    'teardown_cap_stdout': cap_stdout,
                    'teardown_cap_stderr': cap_stderr,
                    'teardown_long_repr_text': long_repr_text,
                    'teardown_outcome': outcome
                })
            process_request('put', result_update_url, json=payload)

    def final_test_batch(self, session: Session):
        if self.report_created:
            exits_status = session.exitstatus
            test_total = session.testscollected
            test_failed = session.testsfailed

            payload = {'total': test_total, 'fail': test_failed}
            final_url = urlparse.urljoin(self.api_host, f'/report/final/{self.session_id}')
            requests.put(final_url, json=payload)


class Hook4ZenTao(object):
    def __init__(self, config):
        self.api_host = config.option.qasite_api_base
        # 初始化是生成session_id，作为一次执行的batch_id
        session_id = int(time.time())
        host_name = socket.gethostname()
        host_ip = socket.gethostbyname(host_name)

        self.da = DataAPI(self.api_host, session_id, agent_name=host_name, agent_ip=host_ip)
        # self.da.submit_test_batch()

    def pytest_runtestloop(self, session: Session):
        """1. 可以在这个阶段提交要执行的全部用例，初始状态为待执行"""
        if self.api_host:
            self.da.submit_test_cases(session)

    def pytest_runtest_logreport(self, report: TestReport):
        """2. 可以记录各个阶段（setup/call/teardown）的结果，及日志输出"""
        if self.api_host:
            self.da.update_test_status(report)

    def pytest_sessionfinish(self, session: Session, exitstatus):
        """3. 提交最终结果"""
        if self.api_host:
            self.da.final_test_batch(session)
